; ------------------------------------------------------------------------
; Define constants

	VARBAS = $8900
	REGBAS = VARBAS + 2
	ZPBAS  = REGBAS + 32

	URXD = $E800
	USTA = $E801
	
; ------------------------------------------------------------------------
; ------------------------------------------------------------------------
		.code
_start:
		sei		; No interrupts
		; Clear screen
		ldx		#0
		lda		#32
l1:		sta		$8000,x
		sta		$8100,x	
		sta		$8200,x
		sta		$8300,x
		inx
		bne		l1
		; Print message
l2:		lda		msg,x
		beq		l3
		and		#63	; Uppercase ASCII -> PETSCII
		sta		$8000+494,x
		inx
		bne		l2
l3:		; start upload
		lda		URXD ; clear UART RX flag
		ldy		#0	
		sty		0	; destination pointer
		sty		1
		; Save npages, nvpages, regs[32], and first 2 bytes of RAM on SNA RAM area
l4:	    bit		USTA 	; UART getchar
		bpl 	l4
		lda		URXD
		sta		VARBAS,y
		iny
		cpy		#36		; total 36 bytes: npages+nvpages+regs(32)+zp[0]+zp[1]
		bne		l4
		; read Memory blocks
		ldy		#2		; start at $0002
		ldx		#0		; block counter: 0=RAM, 1=VRAM
l5:		bit		USTA	; UART getchar
		bpl 	l5
		lda		URXD
		sta		(0),y
		iny
		bne		l5
		inc		1		; next page
		lda		VARBAS,x	; all pages loaded?
		cmp		1
		bne		l5
		lda		#$80	; now point to VRAM
		sta		1
		inx
		cpx		#2
		bne		l5	
	
		; All RAM loaded, now restore register values
		; Zero page restoration 
		lda		ZPBAS	; restore ZP[0]
		sta 	0
		lda		ZPBAS+1	; restore ZP[1]
		sta 	1	
		; VIA reg restoration
		ldx		#14	; IER offset
l6:		lda		REGBAS+8,x
		sta		$E840,x
		dex
		bpl		l6
		; CPU reg restoration 
		ldx 	REGBAS+3	; restore S
		txs
		ldx		REGBAS+1	; restore X
		ldy 	REGBAS+2	; restore Y
		lda 	REGBAS+4	; restore flags (through stack)
		pha
		lda 	REGBAS		; restore Acc
		plp					; restore flags
		jmp		(REGBAS+5)	; restore PC	
	
msg:	.asciiz "UPLOAD SNA"
	
	.export endtxt
endtxt:

